from operator import add
import numpy as np

from datasets.dataset_generation import one_hot_result
from util.db_util import get_specific_bookmaker_odds
from util.util import rps


def generate_season_distribution(data, season):
    result_distribution = [0, 0, 0]
    for idx, match in data[data.season == season].dropna().iterrows():
        result = one_hot_result(match.h_goals, match.a_goals)
        result_distribution = list(map(add, result_distribution, result))
    return [distribution / sum(result_distribution) for distribution in result_distribution]


def generate_season_margin(data, seasons):
    for season in seasons:
        sum_margin = 0
        num_matches = 0
        for idx, match in data[data.season == season].iterrows():
            num_matches += 1
            odds = get_specific_bookmaker_odds(match, "William Hill", dataframe=True)
            margin = sum([1 / odd for odd in odds]) - 1
            sum_margin += margin
        print("Season margin for season {}: {}".format(season, sum_margin / num_matches))


def evaluate_home_victory_rps(test_league_seasons, data):
    for season in test_league_seasons['seasons']:
        rps_scores = []
        probabilities = generate_season_distribution(data, season)
        for idx, match in data[data.season == season].dropna().iterrows():
            result = one_hot_result(match.h_goals, match.a_goals)
            rps_scores.append(rps(probabilities, result))

        rps_scores = np.array([rps_scores])
        min_rps = np.min(rps_scores)
        max_rps = np.max(rps_scores)
        mean_rps = np.mean(rps_scores)

        print("MIN RPS: {}".format(min_rps))
        print("MAX RPS: {}".format(max_rps))
        print("MEAN RPS: {}".format(mean_rps))

    return min_rps, max_rps, mean_rps


def evaluate_odds_rps(test_league_seasons, data):
    for season in test_league_seasons['seasons']:
        rps_scores = []
        for idx, match in data[data.season == season].dropna().iterrows():
            result = one_hot_result(match.h_goals, match.a_goals)
            odds = get_specific_bookmaker_odds(match, "William Hill", dataframe=True)
            probabilities = [1 / odd for odd in odds]
            rps_scores.append(rps(probabilities, result))

        rps_scores = np.array([rps_scores])
        min_rps = np.min(rps_scores)
        max_rps = np.max(rps_scores)
        mean_rps = np.mean(rps_scores)

        print("MIN RPS: {}".format(min_rps))
        print("MAX RPS: {}".format(max_rps))
        print("MEAN RPS: {}".format(mean_rps))

    return min_rps, max_rps, mean_rps


def calculate_ratio_of_favorite_wins(data, seasons):
    for season in seasons:
        num_matches = 0
        num_correct = 0
        for idx, match in data[data.season == season].iterrows():
            num_matches += 1
            odds = get_specific_bookmaker_odds(match, "William Hill", dataframe=True)
            favorite_idx = np.argmin(odds)
            result = one_hot_result(match.h_goals, match.a_goals)

            if favorite_idx == result.index(max(result)):
                num_correct += 1
        print("Accuracy when betting on favorites for season {}: {}".format(season, num_correct / num_matches))
