import math
import pandas
from datetime import datetime

import database.dbmodels as db
from util.util import get_season

leagues = {
    "EPL": "Barclays Premier League",
    "La Liga": "Spanish Primera Division",
    "Ligue 1": "French Ligue 1",
    "Serie A": "Italy Serie A",
    "Bundesliga": "German Bundesliga",
    "Portugal": "",
    "Brazil": "Brasileiro Serie A"
}

def read_spi_and_store_matches_to_db(filepath, league=None, update=True):
    dataframe = pandas.read_csv(filepath, encoding="latin-1")
    if league is not None:
        dataframe = filter_dataframe_by_column_value(dataframe, "league" ,league)
    if update:
        for index, match in dataframe.iterrows():
            update_match_in_database(match)
    else:
        for index, match in dataframe.iterrows():
            insert_match_into_data(match)


def filter_dataframe_by_column_value(dataframe, column, value):
    return dataframe.loc[dataframe[column] == value]

def update_match_in_database(match):
    num = db.Match.select().where(db.Match.season == str(get_season(match['date'])), db.Match.home == match['team1'], db.Match.away == match['team2']).count()
    if num == 0:
       print("match not found")
    else:
        print("Appending to SPI-stats to match")
        if match[3] is None:
            print("Match importance null for match: " +  str(match['date']) + " " + match['team1'] + " vs " + match['team2'])

        query = (db.Match
             .update({db.Match.h_importance: match['importance1'],
                      db.Match.a_importance: match['importance2'],
                      db.Match.h_spi: match['spi1'],
                      db.Match.a_spi: match['spi2'],
                      db.Match.spi_prob_h: match['prob1'],
                      db.Match.spi_prob_a: match['prob2'],
                      db.Match.spi_prob_tie: match['probtie'],
                      db.Match.spi_non_shot_xg_h: match['nsxg1'],
                      db.Match.spi_non_shot_xg_a: match['nsxg2'],
                      db.Match.spi_adj_score_h: match['adj_score1'],
                      db.Match.spi_adj_score_a: match['adj_score2'],
                      db.Match.spi_xg_h: match['xg1'],
                      db.Match.spi_xg_a: match['xg2'],
                      db.Match.h_proj_score: match['proj_score1'],
                      db.Match.a_proj_score: match['proj_score2']
                       })
            .where(db.Match.season == str(get_season(match['date'])), db.Match.home == match['team1'], db.Match.away == match['team2']))
        result = query.execute()
        print("Result: " + str(result))


def insert_match_into_data(match):
    num = db.Match.select().where(db.Match.season == str(get_season(match['date'])), db.Match.home == match['team1'], db.Match.away == match['team2']).count()
    if num == 0:
        print("Appending match: {} - {} - {}".format(match['team1'], match['team2'], match['date']))
        home, _ = db.Team.get_or_create(name=match['team1'])
        away, _ = db.Team.get_or_create(name=match['team2'])
        match, _ = db.Match.get_or_create(
            date=match['date'],
            home=home,
            away=away,
            league=match["league"],
            season=str(get_season(match['date'])),
            h_goals=match["score1"] if not math.isnan(match["score1"]) else None,
            a_goals=match["score2"] if not math.isnan(match["score2"]) else None,
            h_importance=match["importance1"] if not math.isnan(match["importance1"]) else None,
            a_importance=match["importance2"] if not math.isnan(match['importance2']) else None,
            h_spi=match["spi1"] if not math.isnan(match['spi1']) else None,
            a_spi=match["spi2"] if not math.isnan(match['spi2']) else None,
            spi_prob_h=match["prob1"] if not math.isnan(match['prob1']) else None,
            spi_prob_a=match["prob2"] if not math.isnan(match['prob2']) else None,
            spi_prob_tie=match["probtie"] if not math.isnan(match['probtie']) else None,
            spi_non_shot_xg_h=match["nsxg1"] if not math.isnan(match['nsxg1']) else None,
            spi_non_shot_xg_a=match["nsxg2"] if not math.isnan(match['nsxg2']) else None,
            spi_adj_score_h=match["adj_score1"] if not math.isnan(match['adj_score1']) else None,
            spi_adj_score_a=match["adj_score2"] if not math.isnan(match['adj_score2']) else None,
            spi_xg_h=match['xg1'] if not math.isnan(match['xg1']) else None,
            spi_xg_a=match['xg2'] if not math.isnan(match['xg2']) else None,
            h_proj_score=match['proj_score1'] if not math.isnan(match['proj_score1']) else None,
            a_proj_score=match['proj_score2'] if not math.isnan(match['proj_score2']) else None
        )
    else:
        print("Match already exists: {} - {} - {}".format(match['team1'], match['team2'], match['date']))
    return match

#for league in leagues:
 #   read_spi_and_store_existing_matches_to_db("./spi_matches.csv", league=leagues[league])

read_spi_and_store_matches_to_db("./spi_matches.csv",league="EPL", update=True)

