import requests
import pandas
import time
import csv_handler

EPL_TOURNAMENT_ID = "sr:tournament:17"
BASE_URL = "https://api.sportradar.us/soccer-x3/eu/en"
API_KEY = "brma7bqaukzff7ftxaazyh8b"

def get_season_ids(from_year):
    req_url = "/".join([BASE_URL, "tournaments", EPL_TOURNAMENT_ID, "seasons.json"]) + "?api_key=" + API_KEY
    print(req_url)
    seasons = requests.get(req_url).json()["seasons"]
    return {season["year"]: season["id"] for season in seasons if int(season["start_date"][:4]) >= from_year}


def get_season_coverage_levels(seasons):
    coverages = {}
    for season_year, season_id in seasons.items():
        req_url = "/".join([BASE_URL, "tournaments", season_id, "info.json"]) + "?api_key=" + API_KEY
        season_info = requests.get(req_url).json()
        if "season_coverage_info" not in season_info:
            print("No season coverage info for season", season_year)
            continue
        coverages[season_year] = {
            'max': season_info["season_coverage_info"]["max_coverage_level"],
            'played': season_info["season_coverage_info"]["played"],
            'max_covered': season_info["season_coverage_info"]["max_covered"],
            'min': season_info["season_coverage_info"]["min_coverage_level"]
        }
    return coverages

def get_player_rankings():
    req_url = "/".join([BASE_URL, "tournaments", EPL_TOURNAMENT_ID, "leaders.json"]) + "?api_key=" + API_KEY
    print(req_url)
    rankings = requests.get(req_url).json()
    print(rankings.keys())

def get_match_ids(season):
    req_url = "/".join([BASE_URL, "tournaments", season, "schedule.json"])
    req_url += "?api_key=" + API_KEY
    print(req_url)
    data = requests.get(req_url).json()
    events = data["sport_events"]
    ids = []
    for match in events:
        ids.append(match["id"])
    return ids

def get_match_summary(match):
    req_url = "/".join([BASE_URL, "matches", match, "summary.json"]) + "?api_key=" + API_KEY
    while True:
        data = requests.get(req_url)
        if data.status_code != 200:
            print("Throttled, waiting 2 seconds")
            time.sleep(2)
        else:
            return data.json()

def get_tournament_info(id):
    req_url = "/".join([BASE_URL, "tournaments", id, "info.json"]) + "?api_key=" + API_KEY
    return requests.get(req_url).json()

def generate_season_dataset(season_id):
    match_ids = get_match_ids(season_id)

    df = pandas.DataFrame(
        columns=[
            'home_team',
            'away_team',
            'home_poss',
            'home_throwin',
            'home_free_kicks',
            'home_on_target',
            'home_saved',
            'home_corner',
            'home_blocked',
            'home_goal_kicks',
            'home_fouls',
            'home_off_target',
            'home_injuries',
            'home_yellow',
            'home_yellow_red',
            'home_red',
            'away_poss',
            'away_throwin',
            'away_free_kicks',
            'away_on_target',
            'away_saved',
            'away_corner',
            'away_blocked',
            'away_goal_kicks',
            'away_fouls',
            'away_off_target',
            'away_injuries',
            'away_yellow',
            'away_yellow_red',
            'away_red',
            'home_goals',
            'away_goals',
            'outcome'],
        index=[i for i in range(len(match_ids))])

    def get_or_zero(teams, team, key):
        return teams[team]["statistics"][key] if key in teams[team]["statistics"].keys() else 0

    for i,match in enumerate(match_ids):
        print("Getting match {}".format(i))
        data = get_match_summary(match)
        status = data["sport_event_status"]["status"]
        if status != "closed":
            print("{}: {}, skipping".format(data["sport_event"]["id"], status))
            continue
        home_goals = data["sport_event_status"]["home_score"]
        away_goals = data["sport_event_status"]["away_score"]
        if home_goals > away_goals:
            outcome = "H"
        elif away_goals > home_goals:
            outcome = "A"
        else:
            outcome = "D"

        teams = data["statistics"]["teams"]
        try:
            df.loc[i] = pandas.Series({
                "home_team":        teams[0]["abbreviation"],
                "away_team":        teams[1]["abbreviation"],
                'home_poss':        get_or_zero(teams, 0, "ball_possession"),
                'home_throwin':     get_or_zero(teams, 0, "throw_ins"),
                'home_free_kicks':  get_or_zero(teams, 0, "free_kicks"),
                'home_on_target':   get_or_zero(teams, 0, "shots_on_target"),
                'home_saved':       get_or_zero(teams, 0, "shots_saved"),
                'home_corner':      get_or_zero(teams, 0, "corner_kicks"),
                'home_blocked':     get_or_zero(teams, 0, "shots_blocked"),
                'home_goal_kicks':  get_or_zero(teams, 0, "goal_kicks"),
                'home_fouls':       get_or_zero(teams, 0, "fouls"),
                'home_off_target':  get_or_zero(teams, 0, "shots_off_target"),
                'home_injuries':    get_or_zero(teams, 0, "injuries"),
                'home_yellow':      get_or_zero(teams, 0, "yellow_cards"),
                'home_yellow_red':  get_or_zero(teams, 0, "yellow_red_cards"),
                'home_red':         get_or_zero(teams, 0, "red_cards"),
                'away_poss':        get_or_zero(teams, 1, "ball_possession"),
                'away_throwin':     get_or_zero(teams, 1, "throw_ins"),
                'away_free_kicks':  get_or_zero(teams, 1, "free_kicks"),
                'away_on_target':   get_or_zero(teams, 1, "shots_on_target"),
                'away_saved':       get_or_zero(teams, 1, "shots_saved"),
                'away_corner':      get_or_zero(teams, 1, "corner_kicks"),
                'away_blocked':     get_or_zero(teams, 1, "shots_blocked"),
                'away_goal_kicks':  get_or_zero(teams, 1, "goal_kicks"),
                'away_fouls':       get_or_zero(teams, 1, "fouls"),
                'away_off_target':  get_or_zero(teams, 1, "shots_off_target"),
                'away_injuries':    get_or_zero(teams, 1, "injuries"),
                'away_yellow':      get_or_zero(teams, 1, "yellow_cards"),
                'away_yellow_red':  get_or_zero(teams, 1, "yellow_red_cards"),
                'away_red':         get_or_zero(teams, 1, "red_cards"),
                'home_goals':       home_goals,
                'away_goals':       away_goals,
                'outcome':          outcome
            })
        except:
            print(match)
            raise
    return df


def generate_seasonal_lineups_dataset(season_id, headers, player_features):

    dataframe = csv_handler.generate_csv_headers(headers)

    match_ids = get_match_ids(season_id)
    for i,match in enumerate(match_ids):
        print("Getting lineups from match {}".format(i))
        data = get_match_summary(match)
        status = data["sport_event_status"]["status"]
        if status != "closed":
            print("{}: {}, skipping".format(data["sport_event"]["id"], status))
            continue
        teams = data['statistics']['teams']
        match_id = data['sport_event']['id']

        for team in teams:
            for i, player in enumerate(team['players']):
                values = [match_id, team['id']]
                if player['substituted_in'] == 0:
                    values.extend([player[x] for x in player_features])
                dataframe = csv_handler.append_row_to_dataframe(dataframe, values, headers)

    return dataframe


if __name__ == "__main__":
    seasons = get_season_ids(2010)
    time.sleep(2)
    season_2016 = seasons["17/18"]
    coverages = get_season_coverage_levels(seasons)
    print(coverages)

    req_url = "/".join([BASE_URL, "tournaments", seasons["16/17"], "schedule.json"]) + "?api_key=" + API_KEY
    schedule = requests.get(req_url).json()
    print(len(schedule["sport_events"]))

    headers = ["game_id", "team_id", "player_id", "player_name"]
    player_features = ['id', 'name']

    df = generate_seasonal_lineups_dataset(season_2016, headers, player_features)
    df.to_csv("2016-2017-lineups.csv")
