import database.dbmodels as db
import pandas as pd
from datetime import datetime, timedelta

#db.db.drop_tables([db.Bookmaker, db.BookmakerMatchOdds])
#db.db.create_tables([db.Bookmaker, db.BookmakerMatchOdds])

bookies = {
    "B365": db.Bookmaker.get_or_create(name="Bet365")[0],
    "BS": db.Bookmaker.get_or_create(name="Blue Square")[0],
    "BW": db.Bookmaker.get_or_create(name="Bet&Win")[0],
    "GB": db.Bookmaker.get_or_create(name="Gamebookers")[0],
    "IW": db.Bookmaker.get_or_create(name="Interwetten")[0],
    "LB": db.Bookmaker.get_or_create(name="Ladbrokes")[0],
    "PS": db.Bookmaker.get_or_create(name="Pinnacle")[0],
    "SO": db.Bookmaker.get_or_create(name="Sporting Odds")[0],
    "SB": db.Bookmaker.get_or_create(name="Sportingbet")[0],
    "SJ": db.Bookmaker.get_or_create(name="Stan James")[0],
    "SY": db.Bookmaker.get_or_create(name="Stanleybet")[0],
    "VC": db.Bookmaker.get_or_create(name="VC Bet")[0],
    "WH": db.Bookmaker.get_or_create(name="William Hill")[0],
}

base_url = "http://football-data.co.uk/mmz4281/{season}/{league_code}.csv"
league_codes = [
    "E0",   # EPL
    "D1",   # Bundesliga
    "I1",   # Serie A
    "SP1",  # La Liga
    "F1",   # Ligue 1
]
seasons = ["1718", "1617", "1516", "1415"]

DB_TEAM_NAME_LOOKUP = {
    # EPL
    "Man City": "Manchester City",
    "Man United": "Manchester United",
    "Newcastle": "Newcastle United",
    "Wolves": "Wolverhampton Wanderers",
    "West Brom": "West Bromwich Albion",
    "QPR": "Queens Park Rangers",

    # Bundesliga
    "Leverkusen": "Bayer Leverkusen",
    "Hamburg": "Hamburger SV",
    "Hertha": "Hertha Berlin",
    "Stuttgart": "VfB Stuttgart",
    "Hannover": "Hannover 96",
    "Mainz": "Mainz 05",
    "RB Leipzig": "RasenBallsport Leipzig",
    "Dortmund": "Borussia Dortmund",
    "Ein Frankfurt": "Eintracht Frankfurt",
    "M'gladbach": "Borussia M.Gladbach",
    "FC Koln": "FC Cologne",

    # Serie A
    "Milan": "AC Milan",
    "Spal": "SPAL 2013",

    # La Liga
    "Sociedad": "Real Sociedad",
    "Celta": "Celta Vigo",
    "Ath Madrid": "Atletico Madrid",
    "Espanol": "Espanyol",
    "Ath Bilbao": "Athletic Club",
    "Betis": "Real Betis",
    "La Coruna": "Deportivo La Coruna",
    "Sp Gijon": "Sporting Gijon",
    "Vallecano": "Rayo Vallecano",

    # Ligue 1
    "Paris SG": "Paris Saint Germain",
    "St Etienne": "Saint-Etienne",
    "Bastia": "SC Bastia",
    "Ajaccio GFCO": "GFC Ajaccio"
}
for league in league_codes:
    print(league)
    for season in seasons:
        print("Scraping season " + season)
        season_data = pd.read_csv(
            base_url.format(
                season=season,
                league_code=league))
        season_data = season_data.dropna(axis="index", how="all")
        season_year = "20" + season[:2]
        for _, match in season_data.iterrows():
            home = DB_TEAM_NAME_LOOKUP[match.HomeTeam] if match.HomeTeam in DB_TEAM_NAME_LOOKUP else match.HomeTeam
            away = DB_TEAM_NAME_LOOKUP[match.AwayTeam] if match.AwayTeam in DB_TEAM_NAME_LOOKUP else match.AwayTeam
            db_match = list(db.Match.select().where(
                (db.Match.season == season_year) &
                (db.Match.home == home) &
                (db.Match.away == away)
            ))
            if len(db_match) > 1:
                raise RuntimeError("More than one match from db!")
            elif len(db_match) == 0:
                raise RuntimeError("Match not found in db")
            db_match = db_match[0]

            for abbrev, bookie in bookies.items():
                try:
                    home, draw, away = tuple(match[[abbrev + "H", abbrev + "D", abbrev + "A"]])
                    db.BookmakerMatchOdds.get_or_create(match=db_match, bookmaker=bookie, home=home, draw=draw, away=away)
                except Exception as e:
                    continue


