import pickle

import datasets.dataset_generation as dsg


def load_dataframe_data(test_league_seasons, filepath):
    try:
        print("trying to load data from file")
        df = pickle.load(
            open(filepath, "rb"))
        test_data = df[
            (df.league.isin(test_league_seasons['leagues'])) & (df.season.isin(test_league_seasons["seasons"]))]
        data = df.drop(test_data.index)
    except Exception as e:
        print(e)
        print("No data file found, generating...")
        df = dsg.generate_dataframe_dataset(complete_dataset=True)
        test_data = df[
            (df.league.isin(test_league_seasons['leagues'])) & (df.season.isin(test_league_seasons["seasons"]))]
        data = df.drop(test_data.index)
        pickle.dump(df, open(filepath, "wb"))

    return data, test_data


def separate_training_validation_data(data, validation_split):
    validation_end_idx = int(validation_split * len(data))
    return data[validation_end_idx:], data[:validation_end_idx]


def separate_training_validation_data_spi(all_data, fatigue_input, form_input, psychology_input, strength_input,
                                          targets,
                                          validation_split):
    validation_end_idx = int(validation_split * len(all_data))
    validation_fatigue_data = fatigue_input[:validation_end_idx]
    validation_form_data = form_input[:validation_end_idx]
    validation_psychology_data = psychology_input[:validation_end_idx]
    validation_strength_data = strength_input[:validation_end_idx]
    validation_targets = targets[:validation_end_idx]
    training_fatigue_data = fatigue_input[validation_end_idx:]
    training_form_data = form_input[validation_end_idx:]
    training_psychology_data = psychology_input[validation_end_idx:]
    training_strength_data = strength_input[validation_end_idx:]
    training_targets = targets[validation_end_idx:]
    return [validation_fatigue_data, validation_form_data, validation_strength_data, validation_psychology_data], \
           [training_fatigue_data, training_form_data, training_strength_data, training_psychology_data], \
           validation_targets, training_targets


def separate_training_validation_data_exp(all_data, fatigue_input, strength_input, form_input, targets,
                                          validation_split):
    validation_end_idx = int(validation_split * len(all_data))
    validation_fatigue_data = fatigue_input[:validation_end_idx]
    validation_form_data = form_input[:validation_end_idx]
    validation_strength_data = strength_input[:validation_end_idx]
    validation_targets = targets[:validation_end_idx]
    training_fatigue_data = fatigue_input[validation_end_idx:]
    training_form_data = form_input[validation_end_idx:]
    training_strength_data = strength_input[validation_end_idx:]
    training_targets = targets[validation_end_idx:]
    return [validation_fatigue_data, validation_strength_data, validation_form_data], \
           [training_fatigue_data, training_strength_data, training_form_data], \
           validation_targets, training_targets


def separate_test_data(data, test_data_dict):
    test_data = list(filter(
        lambda match: match['season'] in test_data_dict['seasons'] and match['league'] in test_data_dict['leagues'],
        data))
    data = [match for match in data if match not in test_data]
    return data, test_data
