import pickle
from operator import add

from datasets.dataset_generation import one_hot_result
from models import model as md
from datasets.dataloading import separate_test_data
from util.db_util import get_specific_bookmaker_odds
from util.model_input_util import create_spi_input_sub_net, create_feature_single_input, create_exp_input_sub_net
from util.util import load_dict_from_file, rps
import numpy as np


def test_model_dataframe(test_data, model, feature_dict, pi_input=False):
    num_correct = 0
    num_draws_predicted = 0
    rps_scores = []

    for idx, match in test_data.dropna().iterrows():
        if pi_input:
            prediction = model.predict(create_spi_input_sub_net(match, feature_dict))
        else:
            prediction = model.predict([create_feature_single_input(match, feature_dict)])

        rps_scores.append(rps(prediction[0], match["result"]))
        print("{home} {home_score} - {away_score} {away}: {result}. Prediction: {pred}".format(
            home=match["HomeTeam"],
            away=match["AwayTeam"],
            result=match["result"],
            pred=prediction,
            home_score=match["h_goals"],
            away_score=match["a_goals"]
        ))

        if np.argmax(prediction) == 1:
            num_draws_predicted += 1

        if np.argmax(prediction) == match['result'].index(max(match['result'])):
            print("Correct prediction")
            num_correct += 1
        else:
            print("Wrong prediction")

    rps_scores = np.array([rps_scores])
    min_rps = np.min(rps_scores)
    max_rps = np.max(rps_scores)
    mean_rps = np.mean(rps_scores)

    accuracy = num_correct / len(test_data.dropna())
    print("Total accuracy of test-set: {}".format(accuracy))
    print("Number of draws predicted: {}".format(str(num_draws_predicted)))
    print("MIN RPS: {}".format(min_rps))
    print("MAX RPS: {}".format(max_rps))
    print("MEAN RPS: {}".format(mean_rps))
    return min_rps, max_rps, mean_rps, accuracy


def evaluate_validation_exp_data(validation_data, model):
    rps_scores = []
    num_correct = 0
    for idx in range(len(validation_data)):
        prediction = model.predict(np.array([validation_data[idx][0]]))
        rps_scores.append(rps(prediction[0], validation_data[idx][1]))

        print("Result: {result}. Prediction: {pred}".format(
            pred=prediction,
            result=validation_data[idx][1]
        ))
        if np.argmax(prediction) == validation_data[idx][1].argmax():
            print("Correct prediction")
            num_correct += 1

    rps_scores = np.array([rps_scores])
    min_rps = np.min(rps_scores)
    max_rps = np.max(rps_scores)
    mean_rps = np.mean(rps_scores)

    accuracy = num_correct / len(validation_data)

    return min_rps, max_rps, mean_rps, accuracy


def evaluate_validation_pi_data(validation_data, model):
    rps_scores = []
    num_correct = 0
    for idx in range(len(validation_data[0][0])):
        prediction = model.predict([np.expand_dims(validation_data[0][0][idx], 1).transpose(),
                                    np.expand_dims(validation_data[0][1][idx], 1).transpose(),
                                    np.expand_dims(validation_data[0][2][idx], 1).transpose(),
                                    np.expand_dims(validation_data[0][3][idx], 1).transpose()
                                    ])
        rps_scores.append(rps(prediction[0], validation_data[1][idx]))

        print("Result: {result}. Prediction: {pred}".format(
            pred=prediction,
            result=validation_data[1][idx]
        ))
        if np.argmax(prediction) == validation_data[1][idx].argmax():
            print("Correct prediction")
            num_correct += 1

    rps_scores = np.array([rps_scores])
    min_rps = np.min(rps_scores)
    max_rps = np.max(rps_scores)
    mean_rps = np.mean(rps_scores)

    accuracy = num_correct / len(validation_data[0][0])

    return min_rps, max_rps, mean_rps, accuracy


def evaluate_validation_sub_exp_data(validation_data, model):
    rps_scores = []
    num_correct = 0
    for idx in range(len(validation_data[0][0])):
        prediction = model.predict([np.expand_dims(validation_data[0][0][idx], 1).transpose(),
                                    np.expand_dims(validation_data[0][1][idx], 1).transpose(),
                                    np.expand_dims(validation_data[0][2][idx], 1).transpose(),
                                    ])
        rps_scores.append(rps(prediction[0], validation_data[1][idx]))

        print("Result: {result}. Prediction: {pred}".format(
            pred=prediction,
            result=validation_data[1][idx]
        ))
        if np.argmax(prediction) == validation_data[1][idx].argmax():
            print("Correct prediction")
            num_correct += 1

    rps_scores = np.array([rps_scores])
    min_rps = np.min(rps_scores)
    max_rps = np.max(rps_scores)
    mean_rps = np.mean(rps_scores)

    accuracy = num_correct / len(validation_data[0][0])

    return min_rps, max_rps, mean_rps, accuracy




def test_model_peewee(test_data, model, feature_dict):
    num_correct = 0
    num_draws_predicted = 0
    rps_scores = []

    for match in test_data:
        prediction = model.predict([create_feature_single_input(match, feature_dict)])

        rps_scores.append(rps(prediction[0], match["result"]))
        print("{home} {home_score} - {away_score} {away}: {result}. Prediction: {pred}".format(
            home=match["home"],
            away=match["away"],
            result=match["result"],
            pred=prediction,
            home_score=match["h_goals"],
            away_score=match["a_goals"]
        ))

        if np.argmax(prediction) == 1:
            num_draws_predicted += 1

        if np.argmax(prediction) == match['result'].index(max(match['result'])):
            print("Correct prediction")
            num_correct += 1
        else:
            print("Wrong prediction")

    rps_scores = np.array([rps_scores])
    min_rps = np.min(rps_scores)
    max_rps = np.max(rps_scores)
    mean_rps = np.mean(rps_scores)

    accuracy = num_correct / len(test_data)
    print("Total accuracy of test-set: {}".format(accuracy))
    print("Number of draws predicted: {}".format(str(num_draws_predicted)))
    print("MIN RPS: {}".format(min_rps))
    print("MAX RPS: {}".format(max_rps))
    print("MEAN RPS: {}".format(mean_rps))
    return min_rps, max_rps, mean_rps, accuracy


def test_model_peewee_sub_net(test_data, model, feature_dict):
    num_correct = 0
    num_draws_predicted = 0
    rps_scores = []

    for match in test_data:
        input = create_exp_input_sub_net(match, feature_dict)
        prediction = model.predict(input)

        rps_scores.append(rps(prediction[0], match["result"]))
        print("{home} {home_score} - {away_score} {away}: {result}. Prediction: {pred}".format(
            home=match["home"],
            away=match["away"],
            result=match["result"],
            pred=prediction,
            home_score=match["h_goals"],
            away_score=match["a_goals"]
        ))

        if np.argmax(prediction) == 1:
            num_draws_predicted += 1

        if np.argmax(prediction) == match['result'].index(max(match['result'])):
            print("Correct prediction")
            num_correct += 1
        else:
            print("Wrong prediction")

    rps_scores = np.array([rps_scores])
    min_rps = np.min(rps_scores)
    max_rps = np.max(rps_scores)
    mean_rps = np.mean(rps_scores)

    accuracy = num_correct / len(test_data)
    print("Total accuracy of test-set: {}".format(accuracy))
    print("Number of draws predicted: {}".format(str(num_draws_predicted)))
    print("MIN RPS: {}".format(min_rps))
    print("MAX RPS: {}".format(max_rps))
    print("MEAN RPS: {}".format(mean_rps))
    return min_rps, max_rps, mean_rps, accuracy