from time import time

from keras import optimizers, Model, Input
from keras.callbacks import TensorBoard, ModelCheckpoint
from keras.layers import Dense, Dropout, concatenate
from keras.utils import plot_model

from betting import plotting
from betting.betting_evaluator import evaluate_games_weekly
from betting.strategy import VarianceAdjusted, KellyRatio, FixedReturn, FixedBet
from models import model
import datasets.dataloading as dataloading
from datasets.dataloading import separate_training_validation_data_spi
from models.model_evaluator import test_model_dataframe
from models.sub_network_model import SubNetworkModel
from util.model_input_util import create_multiple_inputs_targets_sub_net_dataframe
from util.util import save_dict_to_file


class SubSoccerPowerIndex(SubNetworkModel):

    def __init__(self, sub_input_network_dims, factor_input_sizes,
                 super_network_dims, model_number,dir_name="sub-pi-football", model_name="sub-pi-football", loss="categorical_crossentropy",
                 activation_function="relu", batch_size=256, optimizer='adam', learning_rate=0.0001):
        super().__init__(sub_input_network_dims, factor_input_sizes, super_network_dims,
                         model_number=model_number, dir_name=dir_name, model_name=model_name, loss=loss, activation_function=activation_function,
                         batch_size=batch_size, optimizer=optimizer, learning_rate=learning_rate)
        plot_model(self.model, to_file="model.png")


    @staticmethod
    def get_default_features():
        feature_dict = {
            "fatigue_factors": ['h_days_since_last_exp', 'a_days_since_last_exp'],
            "psychology_factors": ['h_importance', 'a_importance'],
            "strength_factors": ['h_spi', 'a_spi'],
            "form_factors": ['wma_spiFor_Home', 'wma_spiFor_Away']
        }
        feature_list = feature_dict['fatigue_factors'] + feature_dict['form_factors'] + feature_dict['strength_factors'] + \
                       feature_dict['psychology_factors']
        feature_dict['feature_list'] = feature_list
        return feature_dict