import numpy as np


def create_spi_input_sub_net(match, features_dict):
    match_fatigue = np.array([match[feature] for feature in features_dict['fatigue_factors']])
    match_form = np.array([match[feature] for feature in features_dict['form_factors']])
    match_strength = np.array([match[feature] for feature in features_dict['strength_factors']])
    match_psychology = np.array([match[feature] for feature in features_dict['psychology_factors']])
    return [np.expand_dims(match_fatigue, 1).transpose(), np.expand_dims(match_form, 1).transpose(),
            np.expand_dims(match_strength, 1).transpose(), np.expand_dims(match_psychology, 1).transpose()]


def create_exp_input_sub_net(match, features_dict):
    match_fatigue = np.array([match[feature] for feature in features_dict['fatigue_factors']])
    match_form = []
    for feature in features_dict["form_factors"]:
        if isinstance(match[feature], list):
            for value in match[feature]:
                match_form.append(value)
        else:
            match_form.append(match[feature])
    match_form = np.array(match_form)
    match_strength = []
    for feature in features_dict["strength_factors"]:
        if isinstance(match[feature], list):
            for value in match[feature]:
                match_strength.append(value)
        else:
            match_strength.append(match[feature])
    match_strength = np.array(match_strength)
    return [np.expand_dims(match_fatigue, 1).transpose(), np.expand_dims(match_strength, 1).transpose(),
            np.expand_dims(match_form, 1).transpose()]


def create_multiple_inputs_targets_sub_net_dataframe(data, feature_dict, feature_list):
    fatigue_input = np.array([[match[feature] for feature in feature_dict['fatigue_factors']] for idx, match in
                              data[feature_list].dropna().iterrows()])
    psychology_input = np.array([[match[feature] for feature in feature_dict['psychology_factors']] for idx, match in
                                 data[feature_list].dropna().iterrows()])
    strength_input = np.array([[match[feature] for feature in feature_dict['strength_factors']] for idx, match in
                               data[feature_list].dropna().iterrows()])
    form_input = np.array([[match[feature] for feature in feature_dict['form_factors']] for idx, match in
                           data[feature_list].dropna().iterrows()])
    targets = np.array([match["result"] for idx, match in data[feature_list + ['result']].dropna().iterrows()])
    if len(fatigue_input) != len(targets) or len(form_input) != len(targets) \
            or len(strength_input) != len(targets) or len(psychology_input) != len(targets):
        print("Inconsistent input and target vectors. Len not matching. Exiting.")
        raise Exception

    return fatigue_input, psychology_input, strength_input, form_input, targets


def create_multiple_inputs_targets_sub_net_peewee(matches, feature_dict):
    match_fatigue = np.array([[match[feature] for feature in feature_dict['fatigue_factors']] for match in matches])
    form_inputs = []
    for match in matches:
        match_features = []
        for feature in feature_dict["form_factors"]:
            if isinstance(match[feature], list):
                for value in match[feature]:
                    match_features.append(value)
            else:
                match_features.append(match[feature])
        form_inputs.append(np.array(match_features))
    form_inputs = np.array(form_inputs)
    strength_inputs = []
    for match in matches:
        match_features = []
        for feature in feature_dict["strength_factors"]:
            if isinstance(match[feature], list):
                for value in match[feature]:
                    match_features.append(value)
            else:
                match_features.append(match[feature])
        strength_inputs.append(np.array(match_features))
    strength_inputs = np.array(strength_inputs)

    targets = np.array([match["result"] for match in matches])

    if len(match_fatigue) != len(targets) or len(form_inputs) != len(targets) \
            or len(strength_inputs) != len(targets):
        print("Inconsistent input and target vectors. Len not matching. Exiting.")
        raise Exception

    return match_fatigue, strength_inputs, form_inputs, targets




def create_feature_single_input(match, features_dict):
    match_features = []
    for feature in features_dict["feature_list"]:
        if isinstance(match[feature], list):
            for value in match[feature]:
                match_features.append(value)
        else:
            match_features.append(match[feature])
    return np.array([match_features])


def create_features_multiple_input(matches, features_dict):
    all_matches = []
    for match in matches:
        match_features = []
        for feature in features_dict["feature_list"]:
            if isinstance(match[feature], list):
                for value in match[feature]:
                    match_features.append(value)
            else:
                match_features.append(match[feature])
        all_matches.append(np.array(match_features))
    return np.array(all_matches)


def create_features_multiple_input_dataframe(data, feature_list):
    return np.array(
        [[match[feature] for feature in feature_list] for idx, match in data[feature_list].dropna().iterrows()])


def create_multiple_target_output(data):
    return np.array([match["result"] for match in data])


def create_multiple_target_output_dataframe(data, feature_list):
    return np.array([match["result"] for idx, match in data[feature_list + ['result']].dropna().iterrows()])
